/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xlocale.h>

main()
{
    Display *display;
    Window   panel, label, button;
    XEvent   event;
    XFontSet fontset;
    GC       gc;
    char   **miss, *def;
    int      n_miss;

    setlocale(LC_ALL, ""); /* Set language environment (locale) */
    display = XOpenDisplay(NULL); /* Open the display */
    if (XSupportsLocale() == False) exit(1); /* Check if the locale is valid on Xlib */
    XSetLocaleModifiers(""); /* Read the locale modifier */

    /* Create windows at the order of window tree */
    panel = XCreateSimpleWindow(display, RootWindow(display, 0),
                                400, 400, 220, 100, 0, 0, 1);
    label = XCreateSimpleWindow(display, panel, 20, 20, 80, 60, 0, 0, 1);
    button = XCreateSimpleWindow(display, panel, 100, 20, 100, 60, 0, 0, 1);

    /* Preparation for drawing strings and frames */
    gc = XCreateGC(display, panel, NULL, NULL);
    XSetForeground(display, gc, BlackPixel(display, 0));
    fontset = XCreateFontSet(display, "*-14-*", &miss, &n_miss, &def);

    /* Preparation of event processing (Set event masks to be received) */
    XSelectInput(display, label, ExposureMask);
    XSelectInput(display, button, ExposureMask | ButtonPressMask);

    XMapWindow(display, panel); /* Map the windows -> will be shown on screen */
    XMapWindow(display, label);
    XMapWindow(display, button);

    /* Event loop */
    while (1) {
        XNextEvent(display, &event); /* Retrieve event form the que */

        switch (event.type) { /* switch by event type */
        case Expose: /* The window is exposed */
                if (event.xany.window == label) /* check the window ID */
                    /* Redisplay the label string */
                    XmbDrawString(display, label, fontset, gc,
                                  20, 40, "Label", strlen("Label"));
                if (event.xany.window == button) { /* check the window ID */
                    /* Redisplay frame and label for button */
                    XDrawRectangle(display, button, gc, 10, 10, 80, 40);
                    XmbDrawString(display, button, fontset, gc,
                                  30, 40, "Button", strlen("Button"));
                }
                break;
        case ButtonPress: /* The button is pressed */
                if (event.xany.window == button) /* check the window ID */
                    printf("Button is pressed.\n"); /* Respond to user */
                break;
        }
    }
}
