/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>
#include <signal.h>

static int sig_count;  /* The count of the received signal */

/* (2) Signal Handler (just record that signal was received) */
static void sig_handler(sig)
    int sig;
{
    printf("A signal is received.\n");
    sig_count++;  /* count the number of the received signal */
}

/* (4) Function called if signal was arrived */
static void ProcessSignal()
{
    /* Actual signal processing should be done here */
    printf("%d signals were sent.\n", sig_count);
    sig_count = 0;  /* reset the number of the signal */
}

/* Normal callback function */
static void ButtonCB(w, client_data, call_data)
    Widget     w;
    XtPointer  client_data;
    XtPointer  call_data;
{
    int i;

    printf("Button Callback starts.\n");
    for (i = 0; i < 10; i++) {
        printf("Button Callback is processed.\n");
        sleep(1);  /* perform a task for the button */
    }
    printf("Button Callback ends.\n");
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget toplevel, panel, button;
    Arg    al[20];
    int    ac;
    struct sigaction sigact;
    XEvent event;

    XtSetLanguageProc(NULL, NULL, NULL);

    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Sigcustloop", NULL, 0,
                               &argc, argv, NULL, al, ac);

    /* (1) Register signal handler */
    sigact.sa_handler = sig_handler;
    sigemptyset(&sigact.sa_mask);
    sigaddset(&sigact.sa_mask, SIGUSR2);
    sigact.sa_flags = NULL;
    sigaction(SIGUSR1, &sigact, NULL);
    sig_count = 0;

    ac = 0;
    panel = XmCreateRowColumn(toplevel, "panel", al, ac);
    XtManageChild(panel);

    ac = 0;
    button = XmCreatePushButton(panel, "button", al, ac);
    XtManageChild(button);
    XtAddCallback(button, XmNactivateCallback, ButtonCB, NULL);

    XtRealizeWidget(toplevel);

    /* Check signal and event alternately */
    for (;;) {

        /* (3) Check signal arrival */
        if (sig_count > 0)    /* if signal was received */
            ProcessSignal();  /* call a function for signal processing */

        /* (5) Check event arrival */
        else if (XtAppPending(app_context)) {     /* if event was received */
            XtAppNextEvent(app_context, &event);  /* get from que */
            XtDispatchEvent(&event);              /* and dispatch */
        }
    }
}
